extends Control


const target_fps: int = 60

@export var bad_color: Color
@export var warning_color: Color
@export var good_color: Color
@export var fps_counter: Label
@export var frame_time: Label
@export var nodes_counter: Label

var frames_in_time: int = 0
var timer: float = 0

@onready var target_frame_time: float = 1.0 / target_fps * 1000.0


func _process(delta: float) -> void:
	frames_in_time += 1
	timer += delta
	
	if timer >= 1.0:
		var fps: int = roundi(frames_in_time / timer)
		_update_fps_counter(fps)
		_update_nodes_counter()
		frames_in_time = 0
		timer = 0
	
	_update_frame_time(delta * 1000.0)


func _update_fps_counter(fps: int) -> void:
	fps_counter.text = "FPS: %d" % fps
	
	if fps >= target_fps:
		fps_counter.label_settings.font_color = good_color
	elif fps >= roundi(target_fps / 2.0):
		fps_counter.label_settings.font_color = warning_color
	else:
		fps_counter.label_settings.font_color = bad_color


func _update_frame_time(time: float) -> void:
	frame_time.text = "Frame Time: %.1f ms" % time
	
	if time <= target_frame_time:
		frame_time.label_settings.font_color = good_color
	elif time <= roundi(target_frame_time / 2.0):
		frame_time.label_settings.font_color = warning_color
	else:
		frame_time.label_settings.font_color = bad_color


func _update_nodes_counter() -> void:
	nodes_counter.text = "Nodes: %d" % _count_nodes(get_tree().root)


func _count_nodes(node: Node) -> int:
	var count: int = node.get_child_count(true)
	if node.get_child_count(true) > 0:
		for child in node.get_children(true):
			count += _count_nodes(child)
	
	return count
